@@ -760,12 +760,44 @@ Evo.EnsureMainDocumentInitialized = function()
 	Evo.initializeAndPostContentLoaded(null);
 }
 
+Evo.mailDisplaySetIFrameHeight = function(iframe, height, forWidth, force)
+{
+	if (!force && iframe.hasAttribute("x-evo-height-for-width") && iframe.hasAttribute("x-evo-cached-height")) {
+		var heightForWidth = parseInt(iframe.getAttribute("x-evo-height-for-width"));
+		if (heightForWidth == forWidth) {
+			var cachedHeight = parseInt(iframe.getAttribute("x-evo-cached-height"));
+			if (cachedHeight > 0) {
+				iframe.height = cachedHeight;
+				return;
+			}
+		}
+	}
+
+	iframe.setAttribute("x-evo-height-for-width", forWidth);
+	iframe.setAttribute("x-evo-cached-height", height);
+	iframe.height = height;
+}
+
+Evo.mailDisplayGetScrollbarHeight = function()
+{
+	if (Evo.mailDisplayCachedScrollbarHeight != undefined)
+		return Evo.mailDisplayCachedScrollbarHeight;
+
+	var el = document.createElement("div");
+	el.style.cssText = "overflow:scroll; visibility:hidden; position:absolute;";
+	document.body.appendChild(el);
+	Evo.mailDisplayCachedScrollbarHeight = el.offsetHeight - el.clientHeight
+	el.remove();
+
+	return Evo.mailDisplayCachedScrollbarHeight;
+}
+
 Evo.mailDisplayUpdateIFramesHeightRecursive = function(doc)
 {
 	if (!doc)
 		return;
 
-	var ii, iframes;
+	var ii, iframes, force = false;
 
 	iframes = doc.getElementsByTagName("iframe");
 
@@ -777,19 +809,27 @@ Evo.mailDisplayUpdateIFramesHeightRecursive = function(doc)
 	if (!doc.scrollingElement || !doc.defaultView || !doc.defaultView.frameElement)
 		return;
 
-	if (doc.defaultView.frameElement.height == doc.scrollingElement.scrollHeight)
+	if (doc.defaultView.frameElement.height == doc.scrollingElement.scrollHeight) {
 		doc.defaultView.frameElement.height = 10;
-	doc.defaultView.frameElement.height = doc.scrollingElement.scrollHeight + 2 + (doc.scrollingElement.scrollWidth > doc.scrollingElement.clientWidth ? 20 : 0);
+		force = true;
+	}
+
+	Evo.mailDisplaySetIFrameHeight(doc.defaultView.frameElement, doc.scrollingElement.scrollHeight + 2 +
+		(doc.scrollingElement.scrollWidth > doc.scrollingElement.clientWidth ? Evo.mailDisplayGetScrollbarHeight() : 0),
+		doc.scrollingElement.clientWidth, force);
 }
 
 Evo.MailDisplayUpdateIFramesHeight = function()
 {
+	var scrollx = document.defaultView ? document.defaultView.scrollX : -1;
 	var scrolly = document.defaultView ? document.defaultView.scrollY : -1;
 
 	Evo.mailDisplayUpdateIFramesHeightRecursive(document);
 
-	if (scrolly != -1 && document.defaultView.scrollY != scrolly)
-		document.defaultView.scrollTo(0, scrolly);
+	if (scrollx != -1 && scrolly != -1 && (
+	    document.defaultView.scrollX != scrollx ||
+	    document.defaultView.scrollY != scrolly))
+		document.defaultView.scrollTo(scrollx, scrolly);
 
 	Evo.mailDisplayResizeContentToPreviewWidth();
 	Evo.mailDisplayUpdateMagicSpacebarState();
@@ -1179,6 +1219,9 @@ Evo.unsetHTMLColors = function(doc)
 
 Evo.mailDisplaySizeChanged = function(entries, observer)
 {
+	var scrollx = document.defaultView ? document.defaultView.scrollX : -1;
+	var scrolly = document.defaultView ? document.defaultView.scrollY : -1;
+
 	for (const entry of entries) {
 		if (entry.target.ownerDocument.defaultView.frameElement && entry.borderBoxSize?.length > 0) {
 			var value = entry.borderBoxSize[0].blockSize;
@@ -1186,10 +1229,16 @@ Evo.mailDisplaySizeChanged = function(entries, observer)
 			if (value < entry.target.ownerDocument.scrollingElement.scrollHeight)
 				value = entry.target.ownerDocument.scrollingElement.scrollHeight;
 			if (entry.target.ownerDocument.scrollingElement.scrollWidth > entry.target.ownerDocument.scrollingElement.clientWidth)
-				value += 20;
-			entry.target.ownerDocument.defaultView.frameElement.height = value;
+				value += Evo.mailDisplayGetScrollbarHeight();
+			Evo.mailDisplaySetIFrameHeight(entry.target.ownerDocument.defaultView.frameElement, value,
+				entry.target.ownerDocument.scrollingElement.clientWidth, false);
 		}
 	}
+
+	if (scrollx != -1 && scrolly != -1 && (
+	    document.defaultView.scrollX != scrollx ||
+	    document.defaultView.scrollY != scrolly))
+		document.defaultView.scrollTo(scrollx, scrolly);
 }
 
 Evo.MailDisplayBindDOM = function(iframe_id, markCitationColor)
